// Vlad Berteanu - Polytechnic University of Bucharest - 2010

// Implementation of embedded snake game module


#include "snake.h"

coord_t snake[NMAX];
int snake_size;
int snake_dir;
coord_t feed;
int show_score = 0;
byte buffer[20];

// Buttons initialization.
void initButtons() {
	DDRD  &= ~(1 << LEFT_BUTTON);
	PORTD |= 1 << LEFT_BUTTON;
	DDRD  &= ~(1 << UP_BUTTON);
	PORTD |= 1 << UP_BUTTON;
	DDRD  &= ~(1 << RIGHT_BUTTON);
	PORTD |= 1 << RIGHT_BUTTON;
	DDRD  &= ~(1 << DOWN_BUTTON);
	PORTD |= 1 << DOWN_BUTTON;
}

// Check if button was pressed.
void checkButton(int *direction) {

	if ((PIND & (1 << LEFT_BUTTON)) == 0) {
		*direction = LEFT;
		return;
	}
	
	if ((PIND & (1 << UP_BUTTON)) == 0) {
		*direction = UP;
		return;
	}
	
	if ((PIND & (1 << RIGHT_BUTTON)) == 0) {
		*direction = RIGHT;
		return;
	}
	
	if ((PIND & (1 << DOWN_BUTTON)) == 0) {
		*direction = DOWN;
		return;
	}
}


// Snake structure initialization.
void initSnake() {
	int i;

	snake[0].x = 10;
	snake[0].y = 10;
	snake_size = 4;
	snake_dir = RIGHT;
	
	for (i = 1; i < snake_size; i++) {
		snake[i].x = snake[i - 1].x + oriz[RIGHT];
		snake[i].y = snake[i - 1].y + vert[RIGHT];
	}
	
	srand(90);
	feed.x = rand() % 30 + 5;
	feed.y = rand() % 30 + 5;
	
	show_score = 0;
}

// Check if snake died.
int checkSnake() {
	int i, j;
	
	for (i = 0; i < snake_size; i++)
		if (snake[i].x == 1 || snake[i].x == LCD_X_RES ||
		    snake[i].y == 1 || snake[i].y == LCD_Y_RES)
			return DEAD;
			
	for (i = 0; i < snake_size; i++)
		for (j = i + 1; j < snake_size; j++)
		if (snake[i].x == snake[j].x && snake[i].y == snake[j].y)
			return DEAD;
	
	if (snake[snake_size - 1].x == feed.x && 
	    snake[snake_size - 1].y == feed.y) {
		
		snake[snake_size].x = snake[snake_size - 1].x + oriz[snake_dir];
		snake[snake_size].y = snake[snake_size - 1].y + vert[snake_dir];
		snake_size++;
		feed.x = 0;
		show_score = 1;
	}
	
	return ALIVE;
}

// Clear snake.
void clearSnake() {
	int i;
	
	for (i = 0; i < snake_size; i++) 
		LcdPixel(snake[i].x, snake[i].y, PIXEL_OFF);
		
	if (feed.x != 0) 
		LcdPixel(feed.x, feed.y, PIXEL_OFF);
}

// Show snake.
void showSnake() {
	int i;
	
	for (i = 0; i < snake_size; i++)
		LcdPixel(snake[i].x, snake[i].y, PIXEL_ON);
		
	if (feed.x != 0) 
		LcdPixel(feed.x, feed.y, PIXEL_ON);
}

// Move snake according to the current direction.
void moveSnake(int snake_dir) {
	int i;
	coord_t tmp, tmp1;
	
	if (snake_dir == -1) return;

	tmp.x = snake[snake_size - 1].x + oriz[snake_dir];
	tmp.y = snake[snake_size - 1].y + vert[snake_dir];
	
	for (i = snake_size - 1; i >= 0; i--) {
		tmp1 = snake[i];
		snake[i] = tmp;
		tmp = tmp1;
	}

}

// Check if given direction is permitted.
int checkDirection(int old, int current) {
	if (old == RIGHT && current == LEFT) return 0;
	if (old == LEFT  && current == RIGHT) return 0;
	if (old == UP && current == DOWN) return 0;
	if (old == DOWN && current == UP) return 0;
	return 1;
 }


// Snake main function.
int main() {

	int i, dir, dead = 0;

rst:
	dead = 0;
	show_score = 0;
	initSnake();
	initButtons();
	LcdInit();

    // Show game logo.
    LcdContrast(0x7F);
    LcdClear();
    LcdImage(waitImage); 
    LcdUpdate();
    _delay_ms(2500);
	LcdClear();

	showSnake();
		
 	snake_dir = RIGHT;
	while (!dead) {
		LcdClear();
		dir = -1;
		for (i = 0; i < 80; i++) {
			checkButton(&dir);
			if (dir != -1) break;
			_delay_ms(2);
		}
		
		clearSnake();
		if (dir != -1 && checkDirection(snake_dir, dir)) {
			moveSnake(dir);
			snake_dir = dir;
		} else {
			moveSnake(snake_dir);
		}
	
		dead = checkSnake();
		if (dead) continue;
		showSnake();
		if (show_score > 0) {
			LcdGotoXYFont(1,1);
			sprintf(buffer, "Score: %d", snake_size - 4);
			LcdStr(FONT_1X, buffer);
			show_score++;
			if (show_score == 5) show_score = 0;		
		}
		
		LcdUpdate();	
		if (dir != -1)
			_delay_ms(160);
		
		if (rand() % 2 == 1 && feed.x == 0) {
			feed.x = rand() % (LCD_X_RES - 10) + 4;
			feed.y = rand() % (LCD_Y_RES - 10) + 4;
		}
	}
	
	LcdClear();
    LcdImage(overImage); 
    LcdUpdate();
	_delay_ms(2500);
	
	dir = -1;
	do {
		checkButton(&dir);
		
	} while (dir == -1);
	
	goto rst;

	return 0;
}


